@@ -26,6 +26,7 @@
 #include <MsooXmlSchemas.h>
 #include <MsooXmlUtils.h>
 #include <MsooXmlRelationships.h>
+#include <MsooXmlUnits.h>
 
 #include <KoXmlWriter.h>
 #include <KoGenStyles.h>
@@ -38,6 +39,7 @@
 #define MSOOXML_CURRENT_NS "p"
 #define MSOOXML_CURRENT_CLASS PptxXmlSlideReader
 #define BIND_READ_CLASS MSOOXML_CURRENT_CLASS
+#define PPTXXMLSLIDEREADER_CPP
 
 #ifdef __GNUC__
 #warning Using hardcoded presentation:style-name attributes: pr1; pr2
@@ -100,7 +102,6 @@ public:
 PptxXmlSlideReader::PptxXmlSlideReader(KoOdfWriters *writers)
         : MSOOXML::MsooXmlCommonReader(writers)
         , m_context(0)
-        , m_currentDoubleValue(0)
         , m_currentShapeProperties(0)
         , d(new Private)
 {
@@ -109,12 +110,14 @@ PptxXmlSlideReader::PptxXmlSlideReader(KoOdfWriters *writers)
 
 PptxXmlSlideReader::~PptxXmlSlideReader()
 {
+    doneInternal(); // MsooXmlCommonReaderImpl.h
     delete d;
 }
 
 void PptxXmlSlideReader::init()
 {
-    initInternal();
+    initInternal(); // MsooXmlCommonReaderImpl.h
+    initDrawingML();
     m_defaultNamespace = QLatin1String(MSOOXML_CURRENT_NS ":");
 }
 
@@ -134,8 +137,6 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read(MSOOXML::MsooXmlReaderContex
         }*/
     kDebug() << "m_context->slideProperties->shapes.count()" << m_context->slideProperties->shapes.count();
     m_context = 0;
-    if (result == KoFilter::OK)
-        return KoFilter::OK;
     return result;
 }
 
@@ -145,7 +146,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::readInternal()
     if (m_context->type == SlideMaster) {
         //! Clear body pointer for SlideMaster mode: avoid writting to body by mistake in this mode
         d->body = body;
-        body = 0;
+      //  body = 0;
     }
 
     readNext();
@@ -184,6 +185,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::readInternal()
     if (m_context->type == Slide) {
         TRY_READ(sld)
     } else {
+
         TRY_READ(sldMaster)
     }
     kDebug() << "===========finished============";
@@ -227,7 +229,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_sld()
     - sldLayoutIdLst (List of Slide Layouts) §19.3.1.41
     - timing (Slide Timing Information for a Slide Layout) §19.3.1.48
     - transition (Slide Transition for a Slide Layout) §19.3.1.50
-    - txStyles (Slide Master Text Styles) §19.3.1.52
+    - [done] txStyles (Slide Master Text Styles) §19.3.1.52
 */
 //! @todo support all child elements
 KoFilter::ConversionStatus PptxXmlSlideReader::read_sldMaster()
@@ -246,7 +248,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_sldInternal()
     }
 
     if (m_context->type == Slide) {
-        m_currentPageStyle = KoGenStyle(KoGenStyle::StyleDrawingPage, "drawing-page"); // CASE #P109
+        m_currentPageStyle = KoGenStyle(KoGenStyle::DrawingPageStyle, "drawing-page"); // CASE #P109
         m_currentPageStyle.addProperty("presentation:background-visible", true);   // CASE #P111
         m_currentPageStyle.addProperty("presentation:background-objects-visible", true);   // CASE #P112
     }
@@ -264,6 +266,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_sldInternal()
             kDebug() << *this;
             if (isStartElement()) {
                 TRY_READ_IF(cSld)
+                ELSE_TRY_READ_IF(txStyles)
 //! @todo add ELSE_WRONG_FORMAT
             }
             if (m_context->type == Slide) {
@@ -287,7 +290,7 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_sldInternal()
                 //! @todo presentation:use-date-time-name //optional; CASE #P304
                 //! @todo body->addAttribute("presentation:presentation-page-layout-name", ...); //optional; CASE #P308
 
-                const QString currentPageStyleName(mainStyles->lookup(m_currentPageStyle));
+                const QString currentPageStyleName(mainStyles->insert(m_currentPageStyle));
                 body->addAttribute("draw:style-name", currentPageStyleName); // CASE #P302
                 kDebug() << "currentPageStyleName:" << currentPageStyleName;
 
@@ -300,6 +303,190 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_sldInternal()
     return KoFilter::OK;
 }
 
+#undef CURRENT_EL
+#define CURRENT_EL txStyles
+//! txStyles handler (Slide Master Text Styles)
+/*!
+ Parent elements:
+    - [done] sldMaster (§19.3.1.42)
+
+ Child elements:
+    - [done] bodyStyle (Slide Master Body Text Style)   §19.3.1.5
+    - extLst (Extension List)                    §19.2.1.12
+    - [done] otherStyle (Slide Master Other Text Style) §19.3.1.35
+    - [done] titleStyle (Slide Master Title Text Style) §19.3.1.49
+*/
+//! @todo support all child elements
+KoFilter::ConversionStatus PptxXmlSlideReader::read_txStyles()
+{
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        kDebug() << *this;
+        if (isStartElement()) {
+            TRY_READ_IF(bodyStyle)
+            ELSE_TRY_READ_IF(titleStyle)
+            ELSE_TRY_READ_IF(otherStyle)
+//! @todo add ELSE_WRONG_FORMAT
+        }
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL bodyStyle
+//! bodyStyle handler (Slide Master Body Text)
+/*!
+ Parent elements:
+    - [done] txStyles (§19.3.1.52)
+
+ Child elements:
+    - defPPr (Default Paragraph Style)  §21.1.2.2.2
+    - extLst (Extension List)           §20.1.2.2.15
+    - [done] lvl1pPr (List Level 1 Text Style) §21.1.2.4.13
+    - [done] lvl2pPr (List Level 2 Text Style) §21.1.2.4.14
+    - [done] lvl3pPr (List Level 3 Text Style) §21.1.2.4.15
+    - [done] lvl4pPr (List Level 4 Text Style) §21.1.2.4.16
+    - [done] lvl5pPr (List Level 5 Text Style) §21.1.2.4.17
+    - [done] lvl6pPr (List Level 6 Text Style) §21.1.2.4.18
+    - [done] lvl7pPr (List Level 7 Text Style) §21.1.2.4.19
+    - [done] lvl8pPr (List Level 8 Text Style) §21.1.2.4.20
+    - [done] lvl9pPr (List Level 9 Text Style) §21.1.2.4.21
+*/
+//! @todo support all child elements
+KoFilter::ConversionStatus PptxXmlSlideReader::read_bodyStyle()
+{
+    READ_PROLOGUE
+
+    QString currentListStyleName = "bodyList";
+    m_currentListStyle = KoGenStyle(KoGenStyle::ListAutoStyle, "list");
+
+    while (!atEnd()) {
+        readNext();
+        kDebug() << *this;
+        if (isStartElement()) {
+            TRY_READ_IF_NS(a, lvl1pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl2pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl3pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl4pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl5pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl6pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl7pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl8pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl9pPr)
+//! @todo add ELSE_WRONG_FORMAT
+        }
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+
+    mainStyles->insert(m_currentListStyle, currentListStyleName, KoGenStyles::DontAddNumberToName);
+
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL titleStyle
+//! titleStyle handler (Slide Master Title Text)
+/*!
+ Parent elements:
+    - [done] txStyles (§19.3.1.52)
+
+ Child elements:
+    - defPPr (Default Paragraph Style)  §21.1.2.2.2
+    - extLst (Extension List)           §20.1.2.2.15
+    - [done] lvl1pPr (List Level 1 Text Style) §21.1.2.4.13
+    - [done] lvl2pPr (List Level 2 Text Style) §21.1.2.4.14
+    - [done] lvl3pPr (List Level 3 Text Style) §21.1.2.4.15
+    - [done] lvl4pPr (List Level 4 Text Style) §21.1.2.4.16
+    - [done] lvl5pPr (List Level 5 Text Style) §21.1.2.4.17
+    - [done] lvl6pPr (List Level 6 Text Style) §21.1.2.4.18
+    - [done] lvl7pPr (List Level 7 Text Style) §21.1.2.4.19
+    - [done] lvl8pPr (List Level 8 Text Style) §21.1.2.4.20
+    - [done] lvl9pPr (List Level 9 Text Style) §21.1.2.4.21
+*/
+//! @todo support all child elements
+KoFilter::ConversionStatus PptxXmlSlideReader::read_titleStyle()
+{
+    READ_PROLOGUE
+
+    QString currentListStyleName = "titleList";
+    m_currentListStyle = KoGenStyle(KoGenStyle::ListAutoStyle, "list");
+
+    while (!atEnd()) {
+        readNext();
+        kDebug() << *this;
+        if (isStartElement()) {
+            TRY_READ_IF_NS(a, lvl1pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl2pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl3pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl4pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl5pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl6pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl7pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl8pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl9pPr)
+//! @todo add ELSE_WRONG_FORMAT
+        }
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+
+    mainStyles->insert(m_currentListStyle, currentListStyleName, KoGenStyles::DontAddNumberToName);
+
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL otherStyle
+//! otherStyle handler (Slide Master Other Text)
+/*!
+ Parent elements:
+    - [done] txStyles (§19.3.1.52)
+
+ Child elements:
+    - defPPr (Default Paragraph Style)  §21.1.2.2.2
+    - extLst (Extension List)           §20.1.2.2.15
+    - [done] lvl1pPr (List Level 1 Text Style) §21.1.2.4.13
+    - [done] lvl2pPr (List Level 2 Text Style) §21.1.2.4.14
+    - [done] lvl3pPr (List Level 3 Text Style) §21.1.2.4.15
+    - [done] lvl4pPr (List Level 4 Text Style) §21.1.2.4.16
+    - [done] lvl5pPr (List Level 5 Text Style) §21.1.2.4.17
+    - [done] lvl6pPr (List Level 6 Text Style) §21.1.2.4.18
+    - [done] lvl7pPr (List Level 7 Text Style) §21.1.2.4.19
+    - [done] lvl8pPr (List Level 8 Text Style) §21.1.2.4.20
+    - [done] lvl9pPr (List Level 9 Text Style) §21.1.2.4.21
+*/
+//! @todo support all child elements
+KoFilter::ConversionStatus PptxXmlSlideReader::read_otherStyle()
+{
+    READ_PROLOGUE
+
+    QString currentListStyleName = "otherList";
+    m_currentListStyle = KoGenStyle(KoGenStyle::ListAutoStyle, "list");
+
+    while (!atEnd()) {
+        readNext();
+        kDebug() << *this;
+        if (isStartElement()) {
+            TRY_READ_IF_NS(a, lvl1pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl2pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl3pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl4pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl5pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl6pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl7pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl8pPr)
+            ELSE_TRY_READ_IF_NS(a, lvl9pPr)
+//! @todo add ELSE_WRONG_FORMAT
+        }
+        BREAK_IF_END_OF(CURRENT_EL);
+    }
+
+    mainStyles->insert(m_currentListStyle, currentListStyleName, KoGenStyles::DontAddNumberToName);
+
+    READ_EPILOGUE
+}
+
 #undef CURRENT_EL
 #define CURRENT_EL cSld
 //! cSld handler (Common Slide Data)
@@ -359,11 +546,15 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_bg()
 KoFilter::ConversionStatus PptxXmlSlideReader::read_bgPr()
 {
     READ_PROLOGUE
+
+    m_colorType = BackgroundColor;
+
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
         if (isStartElement()) {
             TRY_READ_IF_NS(a, solidFill)
+            ELSE_TRY_READ_IF_NS(a, blip)
 //! @todo add ELSE_WRONG_FORMAT
         }
         BREAK_IF_END_OF(CURRENT_EL);
@@ -482,47 +673,45 @@ KoFilter::ConversionStatus PptxXmlSlideReader::read_txBody()
     m_pPr_lvl = 0;
     const bool isOutline = MSOOXML::Utils::ST_PlaceholderType_to_ODF(m_phType.toLatin1()) == "outline";
 
+    MSOOXML::Utils::XmlWriteBuffer listBuf;
+    body = listBuf.setWriter(body);
+
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
-        if (isStartElement()) {
-            TRY_READ_IF_NS(a, lstStyle)
-//            else TRY_READ_IF_NS(a, p)
+        if (isStartElement()) {            
+	    TRY_READ_IF_NS(a, lstStyle)
             else if (qualifiedName() == QLatin1String("a:p")) {
                 // buffer each text:p, because we have to write after the child elements aregenerated
                 MSOOXML::Utils::XmlWriteBuffer paragraphBuf;
-//                KoXmlWriter *origBody = body;
                 if (isOutline) {
-//                    body = new KoXmlWriter(&paragraphBuf, origBody->indentLevel()+1);
                     body = paragraphBuf.setWriter(body);
                 }
-                TRY_READ(p);
+                TRY_READ(DrawingML_p);
                 if (isOutline) { // CASE #P612
-//                    delete body;
-//                    body = origBody;
-                    body = paragraphBuf.originalWriter();
-                    for (uint i = 0; i <= m_pPr_lvl; i++) {
-                        body->startElement("text:list");
-                        if (i == 0) {
-//! @todo L2 hardcoded
-                            body->addAttribute("text:style-name", "L2");
-                        }
-                        body->startElement("text:list-item");
-                    }
-//                    body->addCompleteElement(&paragraphBuf);
+                     body = paragraphBuf.originalWriter();
                     (void)paragraphBuf.releaseWriter();
-                    for (uint i = 0; i <= m_pPr_lvl; i++) {
-                        body->endElement(); //text:list-item
-                        body->endElement(); //text:list
-                    }
                 }
             }
-//! @todo a:bodyPr
 //! @todo add ELSE_WRONG_FORMAT
         }
         BREAK_IF_END_OF(CURRENT_EL);
     }
 
+    if (m_lstStyleFound) {
+        body = listBuf.originalWriter();
+        //body->startElement("text:list");
+        //const QString currentListStyleName(mainStyles->insert(m_currentListStyle));
+
+        //! @todo currently hardcoded
+        //body->addAttribute("text:style-name", "bodyList");
+        (void)listBuf.releaseWriter();
+        //body->endElement(); // text:list
+    }
+    else {
+        body = listBuf.releaseWriter();
+    }
+
     body->endElement(); // draw:text-box
     READ_EPILOGUE
 }
@@ -570,7 +759,6 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_nvPr()
 // END NAMESPACE p
 
 // BEGIN NAMESPACE a
-//! @todo move to the themes reader or so...
 
 #undef MSOOXML_CURRENT_NS
 #define MSOOXML_CURRENT_NS "a"
@@ -580,122 +768,7 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_nvPr()
 
 #include <MsooXmlCommonReaderImpl.h> // this adds a:p, a:pPr, a:t, a:r, etc.
 
-#undef MSOOXML_CURRENT_NS
-#define MSOOXML_CURRENT_NS "p"
+#define DRAWINGML_NS "a"
+#define DRAWINGML_PIC_NS "p" // DrawingML/Picture
 
 #include <MsooXmlCommonReaderDrawingMLImpl.h> // this adds p:pic, etc.
-
-#undef MSOOXML_CURRENT_NS
-#define MSOOXML_CURRENT_NS "a"
-
-#undef CURRENT_EL
-#define CURRENT_EL solidFill
-//! CASE #P121
-//! @todo support all child elements
-KoFilter::ConversionStatus PptxXmlSlideReader::read_solidFill()
-{
-    READ_PROLOGUE
-    while (!atEnd()) {
-        readNext();
-        kDebug() << *this;
-        if (isStartElement()) {
-            TRY_READ_IF(schemeClr)
-//! @todo add ELSE_WRONG_FORMAT
-        }
-        BREAK_IF_END_OF(CURRENT_EL);
-    }
-    READ_EPILOGUE
-}
-
-#undef CURRENT_EL
-#define CURRENT_EL schemeClr
-//! @todo support all child elements
-KoFilter::ConversionStatus PptxXmlSlideReader::read_schemeClr()
-{
-    READ_PROLOGUE
-    const QXmlStreamAttributes attrs(attributes());
-    READ_ATTR_WITHOUT_NS(val)
-
-    // get color from theme
-    if (m_context->themes->isEmpty())
-        return KoFilter::WrongFormat;
-//! @todo find proper theme, not just any
-    MSOOXML::DrawingMLColorSchemeItemBase *colorItem = 0;
-    if (m_context->type == Slide) {
-        MSOOXML::DrawingMLTheme *theme = m_context->themes->constBegin().value();
-        colorItem = theme->colorScheme.value(val);
-        if (!colorItem || !colorItem->toColorItem())
-            return KoFilter::WrongFormat;
-    }
-    MSOOXML::Utils::DoubleModifier lumMod;
-    MSOOXML::Utils::DoubleModifier lumOff;
-    while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
-        readNext();
-        kDebug() << *this;
-        if (m_context->type == Slide) {
-            if (QUALIFIED_NAME_IS(lumMod)) {
-                m_currentDoubleValue = &lumMod.value;
-                TRY_READ(lumMod);
-                lumMod.valid = true;
-            } else if (QUALIFIED_NAME_IS(lumOff)) {
-                m_currentDoubleValue = &lumOff.value;
-                TRY_READ(lumOff);
-                lumOff.valid = true;
-            }
-        }
-    }
-
-    if (m_context->type == Slide) {
-        QColor col(colorItem->toColorItem()->color);
-        col = MSOOXML::Utils::colorForLuminance(col, lumMod, lumOff);
-        QBrush brush(col, Qt::SolidPattern);
-        KoOdfGraphicStyles::saveOdfFillStyle(m_currentPageStyle, *mainStyles, brush);
-    }
-
-    READ_EPILOGUE
-}
-
-//! 20.1.2.3.20 lumMod (Luminance Modulation)
-//! This element specifies the input color with its luminance modulated by the given percentage.
-//! @todo support all child elements
-#undef CURRENT_EL
-#define CURRENT_EL lumMod
-KoFilter::ConversionStatus PptxXmlSlideReader::read_lumMod()
-{
-    READ_PROLOGUE
-    const QXmlStreamAttributes attrs(attributes());
-    READ_ATTR_WITHOUT_NS(val)
-
-    bool ok;
-    Q_ASSERT(m_currentDoubleValue);
-    *m_currentDoubleValue = MSOOXML::Utils::ST_Percentage_withMsooxmlFix_to_double(val, ok);
-    if (!ok)
-        return KoFilter::WrongFormat;
-
-    readNext();
-    READ_EPILOGUE
-}
-
-//! 20.1.2.3.21 lumOff (Luminance Offset)
-//! This element specifies the input color with its luminance shifted, but with its hue and saturation unchanged.
-//! @todo support all child elements
-#undef CURRENT_EL
-#define CURRENT_EL lumOff
-KoFilter::ConversionStatus PptxXmlSlideReader::read_lumOff()
-{
-    READ_PROLOGUE
-    const QXmlStreamAttributes attrs(attributes());
-    READ_ATTR_WITHOUT_NS(val)
-
-    bool ok;
-    Q_ASSERT(m_currentDoubleValue);
-    *m_currentDoubleValue = MSOOXML::Utils::ST_Percentage_withMsooxmlFix_to_double(val, ok);
-    if (!ok)
-        return KoFilter::WrongFormat;
-
-    readNext();
-    READ_EPILOGUE
-}
-
-// END NAMESPACE a
