@@ -72,7 +72,7 @@ public:
 
     // table of Xformat
     std::vector<XFRecord> xfTable;
-    
+
     // table blib items
     std::vector< MsoDrawingBlibItem* > drawingTable;
 };
@@ -176,8 +176,9 @@ XFRecord GlobalsSubStreamHandler::xformat(unsigned index) const
 
 UString GlobalsSubStreamHandler::valueFormat(unsigned index) const
 {
-    if (index < d->formatsTable.size())
-        return d->formatsTable[index];
+    std::map<unsigned, UString>::iterator it = d->formatsTable.find(index);
+    if (it != d->formatsTable.end())
+        return it->second;
     else
         return UString();
 }
@@ -457,6 +458,7 @@ Format GlobalsSubStreamHandler::convertedFormat(unsigned index) const
     alignment.setRotationAngle(angle);
 
     alignment.setStackedLetters(xf.stackedLetters());
+    alignment.setShrinkToFit(xf.shrinkContent());
 
     format.setAlignment(alignment);
 
@@ -490,7 +492,7 @@ Format GlobalsSubStreamHandler::convertedFormat(unsigned index) const
         pen.color = convertedColor(xf.diagonalColor());
         borders.setBottomLeftBorder(pen);
     }
-    
+
     format.setBorders(borders);
 
     FormatBackground background;
@@ -676,14 +678,23 @@ void GlobalsSubStreamHandler::handleName(NameRecord* record)
     if (!record) return;
 
     d->nameTable.push_back(record->definedName());
-    
+
     if(record->m_formula.id() != FormulaToken::Unused) {
-        FormulaTokens tokens;
-        tokens.push_back(record->m_formula);
-        UString f = decodeFormula(0, 0, false, tokens);
-        if(!f.isEmpty()) {
-            UString n = record->definedName();
-            d->workbook->setNamedArea(n, f);
+        if (record->isBuiltin()) {
+            if (record->definedName() == "_FilterDatabase") {
+                if (record->m_formula.id() == FormulaToken::Area3d) {
+                    std::pair<unsigned, QRect> area = record->m_formula.filterArea3d();
+                    d->workbook->addFilterRange(area.first, area.second);
+                }
+            }
+        } else {
+            FormulaTokens tokens;
+            tokens.push_back(record->m_formula);
+            UString f = decodeFormula(0, 0, false, tokens);
+            if(!f.isEmpty()) {
+                UString n = record->definedName();
+                d->workbook->setNamedArea(record->sheetIndex(), n, f);
+            }
         }
     }
 }
@@ -749,7 +760,7 @@ void GlobalsSubStreamHandler::handleMsoDrawingGroup(MsoDrawingGroupRecord* recor
     if (!record) return;
     printf("GlobalsSubStreamHandler::handleMsoDrawingGroup\n");
     Q_ASSERT(d->drawingTable.size() == 0); // if this asserts then multiple MsoDrawingGroupRecord can exist what we need to handle!
-    d->drawingTable = record->m_items; 
+    d->drawingTable = record->m_items;
 }
 
 MsoDrawingBlibItem* GlobalsSubStreamHandler::drawing(unsigned long pid) const
