@@ -1,5 +1,3 @@
-// 
-
 /*
    This file is part of the KDE project
    Copyright 2001, 2003, 2004 Nicolas GOUTTE <goutte@kde.org>
@@ -16,13 +14,12 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
-#include <qtextcodec.h>
-#include <qradiobutton.h>
-#include <qbuttongroup.h>
+#include <QTextCodec>
+#include <QRadioButton>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -32,27 +29,33 @@
 #include <kcombobox.h>
 #include <kmessagebox.h>
 
-#include <ExportDialogUI.h>
 #include <ExportDialog.h>
 
 AsciiExportDialog :: AsciiExportDialog(QWidget* parent)
-    : KDialogBase(parent, 0, true, i18n("KWord's Plain Text Export Filter"), Ok|Cancel, No, true),
-      m_dialog(new ExportDialogUI(this))
+        : KDialog(parent),
+        m_dialog(new QWidget(this))
 {
+    m_ui.setupUi(m_dialog);
+    m_radioGroup.addButton(m_ui.radioEndOfLineLF);
+    m_radioGroup.addButton(m_ui.radioEndOfLineCRLF);
+    m_radioGroup.addButton(m_ui.radioEndOfLineCR);
 
+    setButtons(Ok | Cancel);
+    setCaption(i18n("KWord's Plain Text Export Filter"));
+    setDefaultButton(KDialog::No);
     kapp->restoreOverrideCursor();
 
     QStringList encodings;
-    encodings << i18n( "Descriptive encoding name", "Recommended ( %1 )" ).arg( "UTF-8" );
-    encodings << i18n( "Descriptive encoding name", "Locale ( %1 )" ).arg( QTextCodec::codecForLocale()->name() );
+    encodings << i18nc("Descriptive encoding name", "Recommended ( %1 )" , QString("UTF-8"));
+    encodings << i18nc("Descriptive encoding name", "Locale ( %1 )" , QString(QTextCodec::codecForLocale()->name()));
     encodings += KGlobal::charsets()->descriptiveEncodingNames();
     // Add a few non-standard encodings, which might be useful for text files
-    const QString description(i18n("Descriptive encoding name","Other ( %1 )"));
-    encodings << description.arg("Apple Roman"); // Apple 
+    const QString description(i18nc("Descriptive encoding name", "Other ( %1 )"));
+    encodings << description.arg("Apple Roman"); // Apple
     encodings << description.arg("IBM 850") << description.arg("IBM 866"); // MS DOS
     encodings << description.arg("CP 1258"); // Windows
 
-    m_dialog->comboBoxEncoding->insertStringList(encodings);
+    m_ui.comboBoxEncoding->addItems(encodings);
 
     setMainWidget(m_dialog);
 
@@ -60,34 +63,30 @@ AsciiExportDialog :: AsciiExportDialog(QWidget* parent)
 
 AsciiExportDialog :: ~AsciiExportDialog(void)
 {
-    kapp->setOverrideCursor(Qt::waitCursor);
+    kapp->setOverrideCursor(Qt::WaitCursor);
 }
 
 QTextCodec* AsciiExportDialog::getCodec(void) const
 {
-    const QString strCodec( KGlobal::charsets()->encodingForName( m_dialog->comboBoxEncoding->currentText() ) );
-    kdDebug(30502) << "Encoding: " << strCodec << endl;
+    const QString strCodec(KGlobal::charsets()->encodingForName(m_ui.comboBoxEncoding->currentText()));
+    kDebug(30502) << "Encoding:" << strCodec;
 
     bool ok = false;
-    QTextCodec* codec = QTextCodec::codecForName( strCodec.utf8() );
+    QTextCodec* codec = QTextCodec::codecForName(strCodec.toUtf8());
 
     // If QTextCodec has not found a valid encoding, so try with KCharsets.
-    if ( codec )
-    {
+    if (codec) {
         ok = true;
-    }
-    else
-    {
-        codec = KGlobal::charsets()->codecForName( strCodec, ok );
+    } else {
+        codec = KGlobal::charsets()->codecForName(strCodec, ok);
     }
 
     // Still nothing?
-    if ( !codec || !ok )
-    {
+    if (!codec || !ok) {
         // Default: UTF-8
-        kdWarning(30502) << "Cannot find encoding:" << strCodec << endl;
+        kWarning(30502) << "Cannot find encoding:" << strCodec;
         // ### TODO: what parent to use?
-        KMessageBox::error( 0, i18n("Cannot find encoding: %1").arg( strCodec ) );
+        KMessageBox::error(0, i18n("Cannot find encoding: %1", strCodec));
         return 0;
     }
 
@@ -97,14 +96,15 @@ QTextCodec* AsciiExportDialog::getCodec(void) const
 QString AsciiExportDialog::getEndOfLine(void) const
 {
     QString strReturn;
-    if (m_dialog->radioEndOfLineLF==m_dialog->buttonGroupEndOfLine->selected())
-        strReturn="\n";
-    else if (m_dialog->radioEndOfLineCRLF==m_dialog->buttonGroupEndOfLine->selected())
-        strReturn="\r\n";
-    else if (m_dialog->radioEndOfLineCR==m_dialog->buttonGroupEndOfLine->selected())
-        strReturn="\r";
+    QAbstractButton* checkedButton = m_radioGroup.checkedButton();
+    if (m_ui.radioEndOfLineLF == checkedButton)
+        strReturn = "\n";
+    else if (m_ui.radioEndOfLineCRLF == checkedButton)
+        strReturn = "\r\n";
+    else if (m_ui.radioEndOfLineCR == checkedButton)
+        strReturn = "\r";
     else
-        strReturn="\n";
+        strReturn = "\n";
 
     return strReturn;
 }
