@@ -25,7 +25,7 @@
 
 #include <kdebug.h>
 
-Paragraph::Paragraph(KoGenStyles* mainStyles, bool inStylesDotXml, bool isHeading, int outlineLevel)
+Paragraph::Paragraph(KoGenStyles* mainStyles, bool inStylesDotXml, bool isHeading, bool inHeaderFooter, int outlineLevel)
         : m_paragraphProperties(0),
         m_paragraphProperties2(0),
         m_odfParagraphStyle(0),
@@ -35,7 +35,11 @@ Paragraph::Paragraph(KoGenStyles* mainStyles, bool inStylesDotXml, bool isHeadin
         m_paragraphStyle2(0),
         m_inStylesDotXml(inStylesDotXml),
         m_isHeading(isHeading),
-        m_outlineLevel(0)
+        m_outlineLevel(0),
+        m_dropCapStatus(NoDropCap),
+        m_inHeaderFooter(inHeaderFooter),
+        m_containsPageNumberField(false),
+	m_combinedCharacters(false)
 {
     kDebug(30513);
     m_mainStyles = mainStyles;
@@ -61,7 +65,9 @@ Paragraph::~Paragraph()
     m_odfParagraphStyle = 0;
 }
 
-void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word97::CHP> chp, QString fontName, const wvWare::StyleSheet& styles)
+void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                             QString fontName, const wvWare::StyleSheet& styles,
+                             bool addCompleteElement)
 {
     // Check for column break in this text string
     int colBreak = text.indexOf(QChar(0xE));
@@ -80,8 +86,13 @@ void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word
         text.remove(QChar(0xE));
     }
 
+    // if it's inner paragraph, push back true
+    // this is an m_textStyles.push_back(NULL) complement if we still need the style applied
+    m_addCompleteElement.push_back(addCompleteElement);
+
     // Add text string to list.
-    m_textStrings.push_back(QString(text));
+    //m_textStrings.push_back(QString(text));
+    m_textStrings.append(QString(text));
 
     // Now find out what style to associate with the string.
 
@@ -111,7 +122,7 @@ void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word
     // chp of the paragraph and the summed chp.
     const wvWare::Style* parentStyle = styles.styleByIndex(msTextStyle->m_std->istdBase);
     if (parentStyle) {
-        applyCharacterProperties(chp, textStyle, m_paragraphStyle, suppresFontSize);
+        applyCharacterProperties(chp, textStyle, m_paragraphStyle, suppresFontSize, m_combinedCharacters);
         //if we have a new font, process that
         const wvWare::Word97::CHP* refChp = &m_paragraphStyle->chp();
         if (!refChp || refChp->ftcAscii != chp->ftcAscii) {
@@ -127,7 +138,7 @@ void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word
         if (!fontName.isEmpty()) {
             textStyle->addProperty(QString("style:font-name"), fontName, KoGenStyle::TextType);
         }
-        applyCharacterProperties(chp, textStyle, m_paragraphStyle, suppresFontSize);
+        applyCharacterProperties(chp, textStyle, m_paragraphStyle, suppresFontSize, m_combinedCharacters);
     }
 
     //add text style to list
@@ -139,7 +150,118 @@ void Paragraph::writeToFile(KoXmlWriter* writer)
     kDebug(30513);
 
     // Set up the paragraph style.
-    applyParagraphProperties(*m_paragraphProperties, m_odfParagraphStyle, m_paragraphStyle);
+    applyParagraphProperties(*m_paragraphProperties, m_odfParagraphStyle, m_paragraphStyle,
+                             m_inHeaderFooter && m_containsPageNumberField, this);
+ 
+    // MS Word puts dropcap characters in its own paragraph with the
+    // rest of the text in the subsequent paragraph. On the other
+    // hand, ODF wants the whole paragraph to be one unit.
+    //
+    // So if this paragraph is only one string and we have a dropcap,
+    // then we should combine this paragraph with the next, and write
+    // them together.  That means we should just return here now so
+    // that the next call to writeToFile() they will be in the same
+    // paragraph.
+    if (m_dropCapStatus == IsDropCapPara) {
+        kDebug(30513) << "returning with drop cap paragraph";
+        return;
+    }
+
+    // If there is a dropcap defined, then write it to the style.
+    if (m_dropCapStatus == HasDropCapIntegrated) {
+        kDebug(30513) << "Creating drop cap style";
+
+        QBuffer buf;
+        buf.open(QIODevice::WriteOnly);
+        KoXmlWriter tmpWriter(&buf, 3);
+        tmpWriter.startElement("style:drop-cap");
+        tmpWriter.addAttribute("style:lines", m_dcs_lines);
+        tmpWriter.addAttributePt("style:distance", m_dropCapDistance);
+        // We have only support for fdct=1, i.e. regular dropcaps.
+        // There is no support for fdct=2, i.e. dropcaps in the margin (ODF doesn't support this).
+        tmpWriter.addAttribute("style:length", m_dcs_fdct > 0 ? 1 : 0);
+        tmpWriter.endElement();//style:drop-cap
+        buf.close();
+
+        QString contents = QString::fromUtf8(buf.buffer(), buf.buffer().size());
+        m_odfParagraphStyle->addChildElement("style:drop-cap", contents);
+
+        m_dropCapStatus = NoDropCap;
+    }
+
+    QString textStyleName;
+    //add paragraph style to the collection and its name to the content
+    kDebug(30513) << "adding paragraphStyle";
+    //add the attribute for our style in <text:p>
+    textStyleName = "P";
+    textStyleName = m_mainStyles->insert(*m_odfParagraphStyle, textStyleName);
+
+    // check if the paragraph is inside frame
+    if (m_paragraphProperties->pap().dxaAbs != 0 || m_paragraphProperties->pap().dyaAbs ) {
+        KoGenStyle userStyle(KoGenStyle::GraphicAutoStyle, "graphic");
+        QString drawStyleName;
+
+        writer->startElement("text:p", false);
+        writer->addAttribute("text:style-name", textStyleName.toUtf8());
+
+        int dxaAbs = 0;
+        const wvWare::Word97::PAP& pap = m_paragraphProperties->pap();
+        //MS-DOC - sprmPDxaAbs - relative horizontal position to anchor
+        if (pap.dxaAbs == -4)   // center
+            userStyle.addProperty("style:horizontal-pos","center");
+        else if (pap.dxaAbs == -8)  // right
+            userStyle.addProperty("style:horizontal-pos","right");
+        else if (pap.dxaAbs == -12)  // inside
+            userStyle.addProperty("style:horizontal-pos","inside");
+        else if (pap.dxaAbs == -16)  // outside
+            userStyle.addProperty("style:horizontal-pos","outside");
+        else //
+            dxaAbs = pap.dxaAbs;
+
+        int dyaAbs = 0;
+        //MS-DOC - sprmPDyaAbs - relative vertical position to anchor
+        if (pap.dyaAbs == -4)   // top
+            userStyle.addProperty("style:vertical-pos","top");
+        else if (pap.dyaAbs == -8)  // middle
+            userStyle.addProperty("style:vertical-pos","middle");
+        else if (pap.dyaAbs == -12)  // bottom
+            userStyle.addProperty("style:vertical-pos","bottom");
+        else if (pap.dyaAbs == -16)  // inside
+            userStyle.addProperty("style:vertical-pos","inline");
+        else if (pap.dyaAbs == -20)  // outside
+            userStyle.addProperty("style:vertical-pos","inline");
+        else //
+            dyaAbs = pap.dyaAbs;
+
+        //MS-DOC - PositionCodeOperand - anchor vertical position
+        if (pap.pcVert == 0)   // margin
+            userStyle.addProperty("style:vertical-rel","page-content");
+        else if (pap.pcVert == 1)  // page
+            userStyle.addProperty("style:vertical-rel","page");
+        else if (pap.pcVert == 2)  // paragraph
+            userStyle.addProperty("style:vertical-rel","paragraph");
+
+        //MS-DOC - PositionCodeOperand - anchor horizontal position
+        if (pap.pcHorz == 0)   // current column
+            userStyle.addProperty("style:horizontal-rel","paragraph");
+        else if (pap.pcHorz == 1)  // margin
+            userStyle.addProperty("style:horizontal-rel","page-content");
+        else if (pap.pcHorz == 2)  // page
+            userStyle.addProperty("style:horizontal-rel","page");
+
+        drawStyleName = "fr";
+        drawStyleName = m_mainStyles->insert(userStyle, drawStyleName);
+        writer->startElement("draw:frame");
+        writer->addAttribute("draw:style-name", drawStyleName.toUtf8());
+        writer->addAttribute("text:anchor-type", "paragraph");
+        if (pap.dxaWidth != 0)
+            writer->addAttributePt("svg:width", (double)pap.dxaWidth/20);
+        if (pap.dyaHeight != 0)
+            writer->addAttributePt("svg:height", (double)pap.dyaHeight/20);
+        writer->addAttributePt("svg:x", (double)dxaAbs/20);
+        writer->addAttributePt("svg:y", (double)dyaAbs/20);
+        writer->startElement("draw:text-box");
+    }
 
     // Open paragraph or heading tag.
     if (m_isHeading) {
@@ -149,20 +271,17 @@ void Paragraph::writeToFile(KoXmlWriter* writer)
         writer->startElement("text:p", false);
     }
 
-    //add paragraph style to the collection and its name to the content
-    kDebug(30513) << "adding paragraphStyle";
-    //add the attribute for our style in <text:p>
-    QString styleName("P");
-    styleName = m_mainStyles->insert(*m_odfParagraphStyle, styleName);
-    writer->addAttribute("text:style-name", styleName.toUtf8());
+    writer->addAttribute("text:style-name", textStyleName.toUtf8());
 
     //just close the paragraph if there's no content
-    if (m_textStrings.empty()) {
+    //if (m_textStrings.empty()) {
+    if (m_textStrings.isEmpty()) {
         writer->endElement(); //text:p
         return;
     }
 
-    //loop through each text strings and styles (equal # of both)
+    // Loop through each text strings and styles (equal # of both) and
+    // write them to the file.
     kDebug(30513) << "writing text spans now";
     QString oldStyleName;
     bool startedSpan = false;
@@ -171,42 +290,56 @@ void Paragraph::writeToFile(KoXmlWriter* writer)
             //if style is null, we have an inner paragraph and add the
             // complete paragraph element to writer
             //need to get const char* from the QString
-            kDebug(30513) << "complete element: " <<
-            m_textStrings[i].toLocal8Bit().constData();
+            kDebug(30513) << "complete element: " << m_textStrings[i].toLocal8Bit().constData();
             writer->addCompleteElement(m_textStrings[i].toLocal8Bit().constData());
         } else {
             //add text style to collection
             //put style into m_mainStyles & get its name
             //kDebug(30513) << m_textStyles[i]->type();
-            styleName = 'T';
-            styleName = m_mainStyles->insert(*m_textStyles[i], styleName);
+            textStyleName = 'T';
+            textStyleName = m_mainStyles->insert(*m_textStyles[i], textStyleName);
 
-            if (oldStyleName != styleName) {
+            if (oldStyleName != textStyleName) {
                 if (startedSpan) {
                     writer->endElement(); //text:span
                     startedSpan = false;
                 }
-                if (styleName != "DefaultParagraphFont") {
+                if (textStyleName != "DefaultParagraphFont") {
                     writer->startElement("text:span");
-                    writer->addAttribute("text:style-name", styleName.toUtf8());
+                    writer->addAttribute("text:style-name", textStyleName.toUtf8());
                     startedSpan = true;
                 }
-                oldStyleName = styleName;
+                oldStyleName = textStyleName;
             }
-            //write text string to writer
-            //now I just need to write the text:span to the header tag
+            // Write text string to writer.
+            // Now I just need to write the text:span to the header tag.
             kDebug(30513) << "Writing \"" << m_textStrings[i] << "\"";
-            writer->addTextSpan(m_textStrings[i]);
+            if (m_addCompleteElement[i] == false) {
+                writer->addTextSpan(m_textStrings[i]);
+            } else {            // special case we need style applied and complete element added
+                writer->addCompleteElement(m_textStrings[i].toLocal8Bit().constData());
+            }
+
             //cleanup
             //delete m_textStyles[i];
             m_textStyles[i] = 0;
         }
     }
-    if (startedSpan) // we wrote a span of text so make sure we close the span tag
+
+    // If we have an unfinished text:span, finish it now.
+    if (startedSpan) {
         writer->endElement(); //text:span
+        startedSpan = false;
+    }
 
     //close the <text:p> or <text:h> tag we opened
     writer->endElement();
+
+    if (m_paragraphProperties->pap().dxaAbs != 0 || m_paragraphProperties->pap().dyaAbs ) {
+        writer->endElement(); //draw:text-box
+        writer->endElement(); // draw:frame
+        writer->endElement(); // close the <text:p>
+    }
 }
 
 void Paragraph::setParagraphStyle(const wvWare::Style* paragraphStyle)
@@ -244,8 +377,10 @@ void Paragraph::openInnerParagraph()
     //m_textStyles2 and m_textStrings2
     m_textStyles2 = m_textStyles;
     m_textStrings2 = m_textStrings;
+    m_addCompleteElement2 = m_addCompleteElement;
     m_textStyles.clear();
     m_textStrings.clear();
+    m_addCompleteElement.clear();
 }
 
 void Paragraph::closeInnerParagraph()
@@ -262,10 +397,13 @@ void Paragraph::closeInnerParagraph()
     m_paragraphProperties2 = 0;
     m_textStyles.clear();
     m_textStrings.clear();
+    m_addCompleteElement.clear();
     m_textStyles = m_textStyles2;
     m_textStrings = m_textStrings2;
+    m_addCompleteElement = m_addCompleteElement2;
     m_textStyles2.clear();
     m_textStrings2.clear();
+    m_addCompleteElement2.clear();
 }
 
 void Paragraph::setParagraphProperties(wvWare::SharedPtr<const wvWare::ParagraphProperties> properties)
@@ -274,7 +412,8 @@ void Paragraph::setParagraphProperties(wvWare::SharedPtr<const wvWare::Paragraph
 }
 
 void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& properties,
-        KoGenStyle* style, const wvWare::Style* parentStyle)
+                                         KoGenStyle* style, const wvWare::Style* parentStyle,
+                                         bool setDefaultAlign, Paragraph *paragraph)
 {
     kDebug(30513);
 
@@ -301,9 +440,12 @@ void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& prop
             style->addProperty("fo:text-align", "justify", KoGenStyle::ParagraphType);
         else //0 = left justify
             style->addProperty("fo:text-align", "start", KoGenStyle::ParagraphType);
+    } else if (setDefaultAlign) {
+        // Set default align for page number field in header or footer
+        kDebug(30513) << "setting default align for page number field in header or footer";
+        style->addProperty("fo:text-align", "center", KoGenStyle::ParagraphType);
     }
 
-    //
     if (!refPap || refPap->fBiDi != pap.fBiDi) {
         if (pap.fBiDi == 1)   //1 = right to left
             style->addProperty("style:writing-mode", "rl-tb", KoGenStyle::ParagraphType);
@@ -425,18 +567,34 @@ void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& prop
         style->addProperty("fo:border-right", Conversion::setBorderAttributes(pap.brcRight), KoGenStyle::ParagraphType);
     }
 
+    // Drop Cap Style (DCS)
     if (!refPap || refPap->dcs.fdct != pap.dcs.fdct || refPap->dcs.lines != pap.dcs.lines) {
+        if (paragraph) {
+            kDebug(30513) << "Processing drop cap";
+            if (paragraph->m_textStrings.size() > 0)
+                kDebug(30513) << "String = """ << paragraph->m_textStrings[0] << """";
+            else
+                kDebug(30513) << "No drop cap string";
+
+            paragraph->m_dropCapStatus = IsDropCapPara;
+            paragraph->m_dcs_fdct  = pap.dcs.fdct;
+            paragraph->m_dcs_lines = pap.dcs.lines;
+            paragraph->m_dropCapDistance = (qreal)pap.dxaFromText / (qreal)20.0;
+        }
+#if 0
         QBuffer buf;
         buf.open(QIODevice::WriteOnly);
         KoXmlWriter tmpWriter(&buf, 3);
-        tmpWriter.startElement("style:drop-cap");
+         tmpWriter.startElement("style:drop-cap");
         tmpWriter.addAttribute("style:lines", pap.dcs.lines);
         tmpWriter.addAttributePt("style:distance", (qreal)pap.dxaFromText / (qreal)20.0);
         tmpWriter.addAttribute("style:length", pap.dcs.fdct > 0 ? 1 : 0);
         tmpWriter.endElement();//style:drop-cap
         buf.close();
+
         QString contents = QString::fromUtf8(buf.buffer(), buf.buffer().size());
         style->addChildElement("style:drop-cap", contents);
+#endif
     }
 
 //TODO introduce diff for tabs too like in: if(!refPap || refPap->fKeep != pap
@@ -490,7 +648,7 @@ void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& prop
     }
 } //end applyParagraphProperties
 
-void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenStyle* style, const wvWare::Style* parentStyle, bool suppressFontSize)
+void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenStyle* style, const wvWare::Style* parentStyle, bool suppressFontSize, bool combineCharacters)
 {
     //if we have a named style, set its CHP as the refChp
     const wvWare::Word97::CHP* refChp;
@@ -629,6 +787,10 @@ void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenSt
             style->addProperty("style:text-outline", "false", KoGenStyle::TextType);
     }
 
+    // if the characters are combined, add proper style
+    if (combineCharacters) {
+        style->addProperty("style:text-combine","letters");
+    }
 
     //dxaSpace = letterspacing in twips
     if (!refChp || refChp->dxaSpace != chp->dxaSpace) {
@@ -636,5 +798,100 @@ void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenSt
         style->addPropertyPt("fo:letter-spacing", value, KoGenStyle::TextType);
     }
     //pctCharwidth = pct stretch doesn't seem to have an ODF ccounterpart but Qt could support it
+
+    //fTNY = 1 when text is vertical
+    if (!refChp || refChp->fTNY != chp->fTNY) {
+        if (chp->fTNY) {
+            style->addProperty("style:text-rotation-angle", 90);
+            if (chp->fTNYCompress) {
+                style->addProperty("style:text-rotation-scale", "fixed");
+            } else {
+                style->addProperty("style:text-rotation-scale", "line-height");
+            }
+        }
+    }
+
+    //wCharScale - MUST be greater than or equal to 1 and less than or equal to 600
+    if (!refChp || refChp->wCharScale != chp->wCharScale) {
+        if (chp->wCharScale) {
+            style->addProperty("style:text-scale", chp->wCharScale);
+        }
+    }
+}
+
+void Paragraph::setCombinedCharacters(bool isCombined)
+{
+    m_combinedCharacters = isCombined;
+}
+
+Paragraph::DropCapStatus Paragraph::dropCapStatus() const
+{
+    return m_dropCapStatus;
+}
+
+void Paragraph::getDropCapData(QString *string, int *type, int *lines, qreal *distance) const
+{
+    // As far as I can see there is only ever a single character as drop cap.
+    *string = m_textStrings[0];
+    *type = m_dcs_fdct;
+    *lines = m_dcs_lines;
+    *distance = m_dropCapDistance;
+}
+
+
+void Paragraph::addDropCap(QString &string, int type, int lines, qreal distance)
+{
+    kDebug(30513) << "combining drop cap paragraph: " << string;
+    if (m_dropCapStatus == IsDropCapPara) 
+        kDebug(30513) << "This paragraph already has a dropcap set!";
+
+    m_dropCapStatus = HasDropCapIntegrated;
+
+    // Get the drop cap data.
+    m_dcs_fdct        = type;
+    m_dcs_lines       = lines;
+    m_dropCapDistance = distance;
+
+    // Add the actual text.
+    // Here we assume that there will only be one text snippet for the drop cap.
+#if 1
+
+#if 1
+    kDebug(30513) << "size: " << m_textStrings.size();
+    m_textStrings[0].prepend(string);
+#else
+    m_textStrings.prepend(string);
+    KoGenStyle* style = 0;
+    m_textStyles.insert(m_textStyles.begin(), style);
+#endif
+
+#else
+    std::vector<QString>            tempStrings;
+    std::vector<const KoGenStyle*>  tempStyles;
+    tempStrings.push_back(dropCapParagraph->m_textStrings[0]);
+    KoGenStyle* style = 0;
+    tempStyles.push_back(style);
+
+    if (dropCapParagraph->m_textStrings.size() > 0) {
+        for (uint i = 0; i < m_textStrings.size(); ++i) {
+            tempStrings.push_back(m_textStrings.[i]);
+            tempStyles.push_back(m_textStyles.[i]);
+        }            
+    }
+
+    m_textStrings = tempStrings;
+    m_textStyles  = tempStyles;
+#endif
+}
+
+
+int Paragraph::strings() const
+{
+    return m_textStrings.size();
+}
+
+QString Paragraph::string(int index) const
+{
+    return m_textStrings[index];
 }
 
