@@ -288,6 +288,7 @@ typedef enum
     sprmCPropRMark = 0xCA57,
     sprmCDispFldRMark = 0xCA62,
     sprmCShd = 0xCA71,
+    sprmCFELayout = 0xCA78,
     sprmPicScale = 0xCE01,
     sprmSOlstAnm = 0xD202,
     sprmSPropRMark = 0xD227,
@@ -320,7 +321,7 @@ typedef enum
     sprmTUndocumented10 = 0xF614,
     sprmTUndocumented11 = 0xF617,
     sprmTUndocumented12 = 0xF618,
-    sprmTUndocumented13 = 0xF661
+    sprmTWidthIndent = 0xF661
 } opcodes;
 
 // The length of the SPRM parameter
@@ -933,9 +934,11 @@ S16 PAP::applyPAPSPRM( const U8* ptr, const Style* style, const StyleSheet* styl
             fNoAutoHyph = *ptr == 1;
             break;
         case SPRM::sprmPWHeightAbs:
-            // Seems to be undocumented...
-            wvlog << "Warning: sprmPWHeightAbs not implemented" << endl;
+        {
+            U16 heightAbsTmp = readU16( ptr );
+            dyaHeight = ( heightAbsTmp & 0xfffe );
             break;
+        }
         case SPRM::sprmPDcs:
             dcs.readPtr( ptr );
             break;
@@ -986,7 +989,7 @@ S16 PAP::applyPAPSPRM( const U8* ptr, const Style* style, const StyleSheet* styl
             break;
         case SPRM::sprmPAnld:
             if ( version == Word8 )
-                anld.readPtr( ptr + 1 ); // variable length, skip lenght byte
+                anld.readPtr( ptr + 1 ); // variable length, skip length byte
             else
                 anld = toWord97( Word95::ANLD( ptr + 1 ) );
             break;
@@ -1411,7 +1414,7 @@ S16 CHP::applyCHPSPRM( const U8* ptr, const Style* paragraphStyle, const StyleSh
             break;
         case SPRM::sprmCHpsNew50:
             if ( *ptr != 2 )
-                wvlog << "Warning: sprmCHpsNew50 has a different lenght than 2" << endl;
+                wvlog << "Warning: sprmCHpsNew50 has a different length than 2" << endl;
             else
                 hps = readU16( ptr + 1 );
             break;
@@ -1520,7 +1523,7 @@ S16 CHP::applyCHPSPRM( const U8* ptr, const Style* paragraphStyle, const StyleSh
         case SPRM::sprmCDispFldRMark:
         {
             if ( *ptr != 39 )
-                wvlog << "Warning: sprmCDispFldRMark has a different lenght than 39" << endl;
+                wvlog << "Warning: sprmCDispFldRMark has a different length than 39" << endl;
             else {
                 fDispFldRMark = *( ptr + 1 ) == 1;
                 ibstDispFldRMark = readS16( ptr + 2 );
@@ -1571,6 +1574,16 @@ S16 CHP::applyCHPSPRM( const U8* ptr, const Style* paragraphStyle, const StyleSh
         case SPRM::sprmCUndocumented1:
         case SPRM::sprmCUndocumented2:
             break;  // They are not documented but they are skipped correctly
+        case SPRM::sprmCFELayout:
+        {
+            //skipping cb
+            U16 ufel = readU16( ptr + sizeof(U8) );
+            fTNY = ufel;
+            //skip to compress;
+            ufel >>= 12;
+            fTNYCompress = ufel;
+            break;
+        }
         default:
             wvlog << "Huh? None of the defined sprms matches 0x" << hex << sprm << dec << "... trying to skip anyway" << endl;
             break;
@@ -1893,6 +1906,10 @@ S16 TAP::applyTAPSPRM( const U8* ptr, const Style* style, const StyleSheet* styl
     U16 sprmLength;
     const U16 sprm( getSPRM( &ptr, version, sprmLength ) );
 
+#ifdef WV2_DEBUG_SPRMS
+    wvlog << "sprm to process: " << hex << sprm << dec << endl;
+    wvlog << "sprmLength: " << hex << sprmLength << dec << endl;
+#endif
     // Is it a TAP sprm? Not really an error if it's none, as all TAP sprms live
     // inside PAP grpprls
     if ( ( ( sprm & 0x1C00 ) >> 10 ) != 5 && sprm != SPRM::sprmPHugePapx && sprm != SPRM::sprmPHugePapx2 ) {
@@ -2304,8 +2321,23 @@ S16 TAP::applyTAPSPRM( const U8* ptr, const Style* style, const StyleSheet* styl
         case SPRM::sprmTUndocumented10:
         case SPRM::sprmTUndocumented11:
         case SPRM::sprmTUndocumented12:
-        case SPRM::sprmTUndocumented13:
             break;
+        case SPRM::sprmTWidthIndent:
+    {
+            //MS-DOC, page 363 of 609
+            const U8 ftsWidth = *ptr;
+            const S16 wWidth = readS16( ptr + 1 );
+
+            //TODO: value is expressed in percentage of page/table, etc.
+            if (ftsWidth == 2) {
+
+        }
+            //value in twipspx
+            else if (ftsWidth == 3) {
+                widthIndent = wWidth;
+        }
+            break;
+    }
         // These are needed in case there are table properties inside the huge papx
         case SPRM::sprmPHugePapx:
         case SPRM::sprmPHugePapx2:
