@@ -51,24 +51,6 @@ KoFilter::ConversionStatus read_fillRect();
 KoFilter::ConversionStatus read_graphic();
 KoFilter::ConversionStatus read_graphicData();
 KoFilter::ConversionStatus read_blipFill();
-KoFilter::ConversionStatus read_anchor();
-KoFilter::ConversionStatus read_positionH();
-KoFilter::ConversionStatus read_positionV();
-enum posOffsetCaller {
-    posOffset_positionH,
-    posOffset_positionV
-};
-KoFilter::ConversionStatus read_posOffset(posOffsetCaller caller);
-enum alignCaller {
-    align_positionH,
-    align_positionV
-};
-KoFilter::ConversionStatus read_align(alignCaller caller);
-KoFilter::ConversionStatus read_inline();
-KoFilter::ConversionStatus read_docPr();
-KoFilter::ConversionStatus read_wrapSquare();
-KoFilter::ConversionStatus read_wrapTight();
-KoFilter::ConversionStatus read_wrapThrough();
 
 KoFilter::ConversionStatus read_DrawingML_p();
 read_p_args m_read_DrawingML_p_args;
@@ -91,6 +73,28 @@ KoFilter::ConversionStatus read_ln();
 KoFilter::ConversionStatus read_srgbClr();
 KoFilter::ConversionStatus read_scrgbClr();
 
+QString m_phType; //! set by read_ph()
+
+KoFilter::ConversionStatus read_spcBef();
+KoFilter::ConversionStatus read_spcPts();
+
+bool m_listStylePropertiesAltered;
+
+KoFilter::ConversionStatus read_buChar();
+KoFilter::ConversionStatus read_buNone();
+KoFilter::ConversionStatus read_buAutoNum();
+KoFilter::ConversionStatus lvlHelper(const QString& level);
+KoFilter::ConversionStatus read_lvl1pPr();
+KoFilter::ConversionStatus read_lvl2pPr();
+KoFilter::ConversionStatus read_lvl3pPr();
+KoFilter::ConversionStatus read_lvl4pPr();
+KoFilter::ConversionStatus read_lvl5pPr();
+KoFilter::ConversionStatus read_lvl6pPr();
+KoFilter::ConversionStatus read_lvl7pPr();
+KoFilter::ConversionStatus read_lvl8pPr();
+KoFilter::ConversionStatus read_lvl9pPr();
+KoFilter::ConversionStatus read_defRPr();
+
 //! Sets style:wrap attribute of style:style/style:graphic-properties element. Used in read_anchor()
 void saveStyleWrap(const char * style);
 
@@ -110,10 +114,12 @@ void readWrap();
 KoFilter::ConversionStatus copyFile(
     const QString& sourceName, const QString& destinationDir, QString& destinationName);
 
-KoGenStyle m_currentDrawStyle; //!< set by read_drawing(), used by read_pic()
+KoGenStyle m_currentDrawStyle; //!< set by read_drawing() and read_object(), used by read_pic() and read_object()
 bool m_drawing_anchor; //! set by read_drawing() to indicate if we have encountered drawing/anchor, used by read_pic()
 bool m_drawing_inline; //! set by read_drawing() to indicate if we have encountered drawing/inline, used by read_pic()
 
+int m_currentListLevel; //! set by drawingML_ppr
+
 int m_svgX; //!< set by read_off()
 int m_svgY; //!< set by read_off()
 int m_svgWidth; //! set by read_ext()
