@@ -104,8 +104,8 @@ public:
     Private() : sheet(0), column(0), row(0) {}
 
     Sheet*  sheet;
-uint    column  : 16; // KS_colMax
-uint    row     : 16; // KS_rowMax
+uint    column  : 17; // KS_colMax
+uint    row     : 17; // KS_rowMax
 };
 
 
@@ -1403,8 +1403,9 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
         } else if (valuetype == "date") {
             QString value = element.attributeNS(KoXmlNS::office, "date-value", QString());
 
-            // "1980-10-15"
-            int year = 0, month = 0, day = 0;
+            // "1980-10-15" or "2001-01-01T19:27:41"
+            int year = 0, month = 0, day = 0, hours = 0, minutes = 0, seconds = 0;
+            bool hasTime = false;
             bool ok = false;
 
             int p1 = value.indexOf('-');
@@ -1413,13 +1414,34 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                 if (ok) {
                     int p2 = value.indexOf('-', ++p1);
                     month = value.mid(p1, p2 - p1).toInt(&ok);
-                    if (ok)
-                        day = value.right(value.length() - p2 - 1).toInt(&ok);
+                    if (ok) {
+                        // the date can optionally have a time attached
+                        int p3 = value.indexOf('T', ++p2);
+                        if (p3 > 0) {
+                            hasTime = true;
+                            day = value.mid(p2, p3 - p2).toInt(&ok);
+                            if (ok) {
+                                int p4 = value.indexOf(':', ++p3);
+                                hours = value.mid(p3, p4 - p3).toInt(&ok);
+                                if (ok) {
+                                    int p5 = value.indexOf(':', ++p4);
+                                    minutes = value.mid(p4, p5 - p4).toInt(&ok);
+                                    if (ok)
+                                        seconds = value.right(value.length() - p5 - 1).toInt(&ok);
+                                }
+                            }
+                        } else {
+                            day = value.right(value.length() - p2).toInt(&ok);
+                        }
+                    }
                 }
             }
 
             if (ok) {
-                setValue(Value(QDate(year, month, day), sheet()->map()->calculationSettings()));
+                if (hasTime)
+                    setValue(Value(QDateTime(QDate(year, month, day), QTime(hours, minutes, seconds)), sheet()->map()->calculationSettings()));
+                else
+                    setValue(Value(QDate(year, month, day), sheet()->map()->calculationSettings()));
 // FIXME Stefan: Should be handled by Value::Format. Verify and remove!
 //Sebsauer: Fixed now. Value::Format handles it correct.
 #if 0
@@ -1690,6 +1712,9 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
 
         // The position is relative to the upper left sheet corner until now. Move it.
         QPointF position = shape->position();
+        // Remember how far we're off from the top-left corner of this cell
+        double offsetX = position.x();
+        double offsetY = position.y();
         for (int col = 1; col < column(); ++col)
             position += QPointF(d->sheet->columnFormat(col)->width(), 0.0);
         for (int row = 1; row < this->row(); ++row)
@@ -1699,7 +1724,9 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
         // The column dimensions are already the final ones, but not the row dimensions.
         // The default height is used for the not yet loaded rows.
         // TODO Stefan: Honor non-default row heights later!
-        QSizeF size = QSizeF(endX, endY);
+        // subtract offset because the accumulated width and height we calculate below starts
+        // at the top-left corner of this cell, but the shape can have an offset to that corner
+        QSizeF size = QSizeF(endX - offsetX, endY - offsetY);
         for (int col = column(); col < endCell.firstRange().left(); ++col)
             size += QSizeF(d->sheet->columnFormat(col)->width(), 0.0);
         for (int row = this->row(); row < endCell.firstRange().top(); ++row)
