@@ -67,6 +67,7 @@ public:
             , userInputStorage(new UserInputStorage())
             , validityStorage(new ValidityStorage(sheet->map()))
             , valueStorage(new ValueStorage())
+            , richTextStorage(new RichTextStorage())
             , undoData(0) {}
 
     Private(const Private& other, Sheet* sheet)
@@ -84,6 +85,7 @@ public:
             , userInputStorage(new UserInputStorage(*other.userInputStorage))
             , validityStorage(new ValidityStorage(*other.validityStorage))
             , valueStorage(new ValueStorage(*other.valueStorage))
+            , richTextStorage(new RichTextStorage(*other.richTextStorage))
             , undoData(0) {}
 
     ~Private() {
@@ -100,6 +102,7 @@ public:
         delete userInputStorage;
         delete validityStorage;
         delete valueStorage;
+        delete richTextStorage;
     }
 
     Sheet*                  sheet;
@@ -116,6 +119,7 @@ public:
     UserInputStorage*       userInputStorage;
     ValidityStorage*        validityStorage;
     ValueStorage*           valueStorage;
+    RichTextStorage*        richTextStorage;
     CellStorageUndoData*    undoData;
 };
 
@@ -142,17 +146,24 @@ CellStorage::~CellStorage()
     delete d;
 }
 
+Sheet* CellStorage::sheet() const
+{
+    return d->sheet;
+}
+
 void CellStorage::take(int col, int row)
 {
     Formula oldFormula;
     QString oldLink;
     QString oldUserInput;
     Value oldValue;
+    QSharedPointer<QTextDocument> oldRichText;
 
     oldFormula = d->formulaStorage->take(col, row);
     oldLink = d->linkStorage->take(col, row);
     oldUserInput = d->userInputStorage->take(col, row);
     oldValue = d->valueStorage->take(col, row);
+    oldRichText = d->richTextStorage->take(col, row);
 
     if (!d->sheet->map()->isLoading()) {
         // Trigger a recalculation of the consuming cells.
@@ -166,6 +177,7 @@ void CellStorage::take(int col, int row)
         d->undoData->links      << qMakePair(QPoint(col, row), oldLink);
         d->undoData->userInputs << qMakePair(QPoint(col, row), oldUserInput);
         d->undoData->values     << qMakePair(QPoint(col, row), oldValue);
+        d->undoData->richTexts  << qMakePair(QPoint(col, row), oldRichText);
     }
 }
 
@@ -357,6 +369,24 @@ void CellStorage::setUserInput(int column, int row, const QString& userInput)
         d->undoData->userInputs << qMakePair(QPoint(column, row), old);
 }
 
+QSharedPointer<QTextDocument> CellStorage::richText(int column, int row) const
+{
+    return d->richTextStorage->lookup(column, row);
+}
+
+void CellStorage::setRichText(int column, int row, QSharedPointer<QTextDocument> text)
+{
+    QSharedPointer<QTextDocument> old;
+    if (text.isNull())
+        old = d->richTextStorage->take(column, row);
+    else
+        old = d->richTextStorage->insert(column, row, text);
+
+    // recording undo?
+    if (d->undoData && text != old)
+        d->undoData->richTexts << qMakePair(QPoint(column, row), old);
+}
+
 Validity CellStorage::validity(int column, int row) const
 {
     return d->validityStorage->contains(QPoint(column, row));
@@ -599,6 +629,7 @@ void CellStorage::insertColumns(int position, int number)
     QList< QPair<QRectF, QString> > namedAreas = d->namedAreaStorage->insertColumns(position, number);
     QList< QPair<QRectF, SharedSubStyle> > styles = d->styleStorage->insertColumns(position, number);
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->insertColumns(position, number);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->insertColumns(position, number);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->insertColumns(position, number);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->insertColumns(position, number);
     // recording undo?
@@ -616,6 +647,7 @@ void CellStorage::insertColumns(int position, int number)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -656,6 +688,7 @@ void CellStorage::removeColumns(int position, int number)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->removeColumns(position, number);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->removeColumns(position, number);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->removeColumns(position, number);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->removeColumns(position, number);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -671,6 +704,7 @@ void CellStorage::removeColumns(int position, int number)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -710,6 +744,7 @@ void CellStorage::insertRows(int position, int number)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->insertRows(position, number);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->insertRows(position, number);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->insertRows(position, number);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->insertRows(position, number);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -725,6 +760,7 @@ void CellStorage::insertRows(int position, int number)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -765,6 +801,7 @@ void CellStorage::removeRows(int position, int number)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->removeRows(position, number);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->removeRows(position, number);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->removeRows(position, number);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->removeRows(position, number);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -780,6 +817,7 @@ void CellStorage::removeRows(int position, int number)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -820,6 +858,7 @@ void CellStorage::removeShiftLeft(const QRect& rect)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->removeShiftLeft(rect);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->removeShiftLeft(rect);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->removeShiftLeft(rect);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->removeShiftLeft(rect);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -835,6 +874,7 @@ void CellStorage::removeShiftLeft(const QRect& rect)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -874,6 +914,7 @@ void CellStorage::insertShiftRight(const QRect& rect)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->insertShiftRight(rect);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->insertShiftRight(rect);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->insertShiftRight(rect);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->insertShiftRight(rect);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -889,6 +930,7 @@ void CellStorage::insertShiftRight(const QRect& rect)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -929,6 +971,7 @@ void CellStorage::removeShiftUp(const QRect& rect)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->removeShiftUp(rect);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->removeShiftUp(rect);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->removeShiftUp(rect);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->removeShiftUp(rect);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -944,6 +987,7 @@ void CellStorage::removeShiftUp(const QRect& rect)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -983,6 +1027,7 @@ void CellStorage::insertShiftDown(const QRect& rect)
     QVector< QPair<QPoint, QString> > userInputs = d->userInputStorage->insertShiftDown(rect);
     QList< QPair<QRectF, Validity> > validities = d->validityStorage->insertShiftDown(rect);
     QVector< QPair<QPoint, Value> > values = d->valueStorage->insertShiftDown(rect);
+    QVector< QPair<QPoint, QSharedPointer<QTextDocument> > > richTexts = d->richTextStorage->insertShiftDown(rect);
     // recording undo?
     if (d->undoData) {
         d->undoData->bindings   << bindings;
@@ -998,6 +1043,7 @@ void CellStorage::insertShiftDown(const QRect& rect)
         d->undoData->userInputs << userInputs;
         d->undoData->validities << validities;
         d->undoData->values     << values;
+        d->undoData->richTexts  << richTexts;
     }
 
     // Trigger a dependency update of the cells, which have a formula. (new positions)
@@ -1139,7 +1185,6 @@ Cell CellStorage::prevInRow(int col, int row, Visiting visiting) const
 int CellStorage::columns() const
 {
     int max = 0;
-    max = qMax(max, d->bindingStorage->usedArea().right());
     max = qMax(max, d->commentStorage->usedArea().right());
     max = qMax(max, d->conditionsStorage->usedArea().right());
     max = qMax(max, d->fusionStorage->usedArea().right());
@@ -1148,13 +1193,16 @@ int CellStorage::columns() const
     max = qMax(max, d->formulaStorage->columns());
     max = qMax(max, d->linkStorage->columns());
     max = qMax(max, d->valueStorage->columns());
+
+    // don't include bindings cause the bindingStorage does only listen to all cells in the sheet.
+    //max = qMax(max, d->bindingStorage->usedArea().right());
+
     return max;
 }
 
 int CellStorage::rows() const
 {
     int max = 0;
-    max = qMax(max, d->bindingStorage->usedArea().bottom());
     max = qMax(max, d->commentStorage->usedArea().bottom());
     max = qMax(max, d->conditionsStorage->usedArea().bottom());
     max = qMax(max, d->fusionStorage->usedArea().bottom());
@@ -1163,6 +1211,10 @@ int CellStorage::rows() const
     max = qMax(max, d->formulaStorage->rows());
     max = qMax(max, d->linkStorage->rows());
     max = qMax(max, d->valueStorage->rows());
+
+    // don't include bindings cause the bindingStorage does only listen to all cells in the sheet.
+    //max = qMax(max, d->bindingStorage->usedArea().bottom());
+
     return max;
 }
 
@@ -1255,6 +1307,8 @@ void CellStorage::undo(CellStorageUndoData* data)
         setValue(data->values[i].first.x(), data->values[i].first.y(), data->values[i].second);
     for (int i = 0; i < data->userInputs.count(); ++i)
         setUserInput(data->userInputs[i].first.x(), data->userInputs[i].first.y(), data->userInputs[i].second);
+    for (int i = 0; i < data->richTexts.count(); ++i)
+        setRichText(data->richTexts[i].first.x(), data->richTexts[i].first.y(), data->richTexts[i].second);
     for (int i = 0; i < data->links.count(); ++i)
         setLink(data->links[i].first.x(), data->links[i].first.y(), data->links[i].second);
     for (int i = 0; i < data->fusions.count(); ++i)
