@@ -48,6 +48,8 @@
 #include "dockers/KWStatisticsDocker.h"
 #include "commands/KWFrameCreateCommand.h"
 #include "commands/KWCreateOutlineCommand.h"
+#include "commands/KWClipFrameCommand.h"
+#include "commands/KWRemoveFrameClipCommand.h"
 
 // koffice libs includes
 #include <KoCopyController.h>
@@ -97,10 +99,17 @@
 
 static KWFrame *frameForShape(KoShape *shape)
 {
-    KoShape *parent = shape;
-    while (parent->parent())
-        parent = parent->parent();
-    return dynamic_cast<KWFrame*>(parent->applicationData());
+    while (shape->parent()) {
+        shape = shape->parent();
+    }
+    KWFrame *answer = dynamic_cast<KWFrame*>(shape->applicationData());
+    if (answer == 0) { // this may be a clipping shape containing the frame-shape
+        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
+        if (container && container->childCount() == 1) {
+            answer = dynamic_cast<KWFrame*>(container->childShapes()[0]->applicationData());
+        }
+    }
+    return answer;
 }
 
 KWView::KWView(const QString &viewMode, KWDocument *document, QWidget *parent)
@@ -425,12 +434,19 @@ if (false) { // TODO move this to the text tool as soon as  a) the string freeze
     action->setWhatsThis(i18n("Create a copy of the current frame, that remains linked to it. This means they always show the same contents: modifying the contents in such a frame will update all its linked copies."));
     connect(action, SIGNAL(triggered()), this, SLOT(createLinkedFrame()));
 
-    action = new KAction(i18n("Create custom outline"), this);
+    action = new KAction(i18n("Create Custom Outline"), this);
     actionCollection()->addAction("create_custom_outline", action);
     action->setToolTip(i18n("Create a custom vector outline that text will run around"));
     action->setWhatsThis(i18n("Text normally runs around the content of a shape, when you want a custom outline that is independent of the content you can create one and alter it with the vector tools"));
     connect(action, SIGNAL(triggered()), this, SLOT(createCustomOutline()));
 
+    action = new KAction(i18n("Create Frame-clip"), this);
+    actionCollection()->addAction("create_clipped_frame", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(createFrameClipping()));
+
+    action = new KAction(i18n("Remove Frame-clip"), this);
+    actionCollection()->addAction("remove_clipped_frame", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(removeFrameClipping()));
 
     //------------------------ Settings menu
     action = new KToggleAction(i18n("Status Bar"), this);
@@ -1283,6 +1299,36 @@ void KWView::createCustomOutline()
     KoToolManager::instance()->switchToolRequested("PathToolFactoryId");
 }
 
+void KWView::createFrameClipping()
+{
+    QSet<KWFrame *> clipFrames;
+    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+        KWFrame *frame = frameForShape(shape);
+        Q_ASSERT(frame);
+        if (frame->shape()->parent() == 0)
+            clipFrames << frame;
+    }
+    if (!clipFrames.isEmpty()) {
+        KWClipFrameCommand *cmd = new KWClipFrameCommand(clipFrames.toList(), m_document);
+        m_document->addCommand(cmd);
+    }
+}
+
+void KWView::removeFrameClipping()
+{
+    QSet<KWFrame *> unClipFrames;
+    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+        KWFrame *frame = frameForShape(shape);
+        Q_ASSERT(frame);
+        if (frame->shape()->parent())
+            unClipFrames << frame;
+    }
+    if (!unClipFrames.isEmpty()) {
+        KWRemoveFrameClipCommand *cmd = new KWRemoveFrameClipCommand(unClipFrames.toList(), m_document);
+        m_document->addCommand(cmd);
+    }
+}
+
 void KWView::insertImage()
 {
     KoShape *shape = KoImageSelectionWidget::selectImageShape(m_document->resourceManager(), this);
