@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2006, 2008 Casper Boemann <cbr@boemann.dk>
-   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
    Copyright (C) 2007-2009 Jan Hambrecht <jaham@gmx.net>
 
    This library is free software; you can redistribute it and/or
@@ -54,7 +54,7 @@ class KoCanvasBase;
 class KoShapeLoadingContext;
 class KoGenStyle;
 class KoShapeControllerBase;
-class KoDataCenter;
+class KoDataCenterBase;
 class KoShapeShadow;
 class KoEventAction;
 class KoShapePrivate;
@@ -160,7 +160,7 @@ public:
      * @param painter used for painting the shape
      * @param converter to convert between internal and view coordinates.
      * @param canvas the canvas that requested this paint.  This can be used to retrieve canvas specific properties
-     *      like selection and get a reference to the KoCanvasResourceProvider.
+     *      like selection and get a reference to the KoResourceManager.
      */
     virtual void paintDecorations(QPainter &painter, const KoViewConverter &converter, const KoCanvasBase *canvas);
 
@@ -199,13 +199,6 @@ public:
      */
     void saveOdfCommonChildElements(KoShapeSavingContext &context) const;
 
-    /**
-     * After the shape has been created this method is called so it can get access to any DataCenter it
-     * might want.
-     * The default implementation does nothing.
-     */
-    virtual void init(const QMap<QString, KoDataCenter*> &dataCenterMap);
-
     /**
      * @brief Scale the shape using the zero-point which is the top-left corner.
      * @see position()
@@ -256,7 +249,7 @@ public:
      *
      * The size is in shape coordinates.
      *
-     * @return the size of the shape as set by resize()
+     * @return the size of the shape as set by setSize()
      */
     virtual QSizeF size() const;
 
@@ -610,7 +603,7 @@ public:
 
     /**
      * Return the Id of this shape, identifying the type of shape by the id of the factory.
-     * @see KoShapeFactory::shapeId()
+     * @see KoShapeFactoryBase::shapeId()
      * @return the id of the shape-type
      */
     QString shapeId() const;
@@ -618,7 +611,7 @@ public:
     /**
      * Set the Id of this shape.  A shapeFactory is expected to set the Id at creation
      * so applications can find out what kind of shape this is.
-     * @see KoShapeFactory::shapeId()
+     * @see KoShapeFactoryBase::shapeId()
      * @param id the ID from the factory that created this shape
      */
     void setShapeId(const QString &id);
@@ -669,7 +662,7 @@ public:
      * is a notable exclusion.
      * @param shape the shape to use as original
      */
-    virtual void copySettings(const KoShape *shape);
+    void copySettings(const KoShape *shape);
 
     /**
      * Convenience method that allows people implementing paint() to use the shape
@@ -760,16 +753,22 @@ public:
 
     /**
      * Adds a shape which depends on this shape.
+     * Making a shape dependent on this one means it will get shapeChanged() called
+     * on each update of this shape.
      *
      * If this shape already depends on the given shape, establishing the
      * dependency is refused to prevent circular dependencies.
      *
      * @param shape the shape which depends on this shape
-     * @return true if dependency could be established, else false
+     * @return true if dependency could be established, otherwise false
+     * @see removeDependee(), hasDependee()
      */
     bool addDependee(KoShape *shape);
 
-    /// Removes as shape depending on this shape
+    /**
+     * Removes as shape depending on this shape.
+     * @see addDependee(), hasDependee()
+     */
     void removeDependee(KoShape *shape);
 
     /// Returns if the given shape is dependent on this shape
@@ -842,6 +841,24 @@ public:
      */
     bool collisionDetection();
 
+    /**
+     * Return the tool delegates for this shape.
+     * In Flake a shape being selected will cause the tool manager to make available all tools that
+     * can edit the selected shapes.  In some cases selecting one shape should allow the tool to
+     * edit a related shape be available too.  The tool delegates allows this to happen by taking
+     * all the shapes in the set into account on tool selection.
+     * Notice that if the set is non-empty 'this' shape is no longer looked at. You can choose
+     * to add itself to the set too.
+     */
+    QSet<KoShape*> toolDelegates() const;
+
+    /**
+     * Set the tool delegates.
+     * @param delegates the new delegates.
+     * @see toolDelegates()
+     */
+    void setToolDelegates(const QSet<KoShape*> &delegates);
+
     /**
      * \internal
      * Returns the private object for use withing the flake lib
@@ -912,20 +929,11 @@ protected:
      */
     virtual void loadStyle(const KoXmlElement &element, KoShapeLoadingContext &context);
 
-    /// Loads the fill style
-    //QBrush loadOdfFill( const KoXmlElement & element, KoShapeLoadingContext & context );
-    KoShapeBackground *loadOdfFill(const KoXmlElement &element, KoShapeLoadingContext &context);
-
     /// Loads the stroke style
-    KoShapeBorderModel *loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context);
+    KoShapeBorderModel *loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const;
 
     /// Loads the shadow style
-    KoShapeShadow *loadOdfShadow(const KoXmlElement &element, KoShapeLoadingContext &context);
-
-    /**
-     * Fills the style stack and returns the value of the given style property (e.g fill, stroke).
-     */
-    QString getStyleProperty(const char *property, const KoXmlElement &element, KoShapeLoadingContext &context);
+    KoShapeBackground *loadOdfFill(KoShapeLoadingContext &context) const;
 
     /* ** end loading saving */
 
