@@ -38,7 +38,10 @@ public:
     /**
      * Fills the style stack and returns the value of the given style property (e.g fill, stroke).
      */
-    static QString getStyleProperty(const char *property, const KoXmlElement &element, KoShapeLoadingContext &context);
+    static QString getStyleProperty(const char *property, KoShapeLoadingContext &context);
+
+    /// Loads the shadow style
+    KoShapeShadow *loadOdfShadow(KoShapeLoadingContext &context) const;
 
     /// calls update on the shape where the border is.
     void updateBorder();
@@ -49,7 +52,7 @@ public:
 
     QMatrix localMatrix; ///< the shapes local transformation matrix
 
-    QVector<QPointF> connectors; // in pt
+    QVector<QPointF> connectors; ///< glue points in percent of size [0..1]
 
     KoShapeContainer *parent;
     QSet<KoShapeManager *> shapeManagers;
@@ -65,7 +68,7 @@ public:
     QMap<QByteArray, QString> additionalStyleAttributes;
     QSet<KoEventAction *> eventActions; ///< list of event actions the shape has
     KoFilterEffectStack *filterEffectStack; ///< stack of filter effects applied to the shape
-    qreal transparency; ///< the shapes tranparency
+    qreal transparency; ///< the shapes transparency
 
     static const int MaxZIndex = 32767;
     int zIndex : 16; // keep maxZIndex in sync!
