@@ -25,6 +25,7 @@
 #include "KoToolManager_p.h"
 #include "KoToolRegistry.h"
 #include "KoToolProxy.h"
+#include "KoToolProxy_p.h"
 #include "KoSelection.h"
 #include "tools/KoCreatePathToolFactory.h"
 #include "tools/KoCreateShapesToolFactory.h"
@@ -393,6 +394,7 @@ void KoToolManager::Private::detachCanvas(KoCanvasController *controller)
             inputDevice = canvasData->inputDevice;
             canvasData->canvas->priv()->activate();
         } else {
+            canvasData->canvas->setToolOptionWidgets(QMap<QString, QWidget *>());
             // as a last resort just set a blank one
             canvasData = 0;
             // and stop the event filter
@@ -435,7 +437,7 @@ void KoToolManager::Private::attachCanvas(KoCanvasController *controller)
 
     KoToolProxy *tp = proxies[controller->canvas()];
     if (tp)
-        tp->setCanvasController(controller);
+        tp->priv()->setCanvasController(controller);
 
     if (cd->activeTool == 0) {
         // no active tool, so we activate the highest priority main tool
@@ -650,7 +652,7 @@ void KoToolManager::Private::registerToolProxy(KoToolProxy *proxy, KoCanvasBase
     proxies.insert(canvas, proxy);
     foreach(KoCanvasController *controller, canvasses.keys()) {
         if (controller->canvas() == canvas) {
-            proxy->setCanvasController(controller);
+            proxy->priv()->setCanvasController(controller);
             break;
         }
     }
@@ -796,7 +798,7 @@ KoCreateShapesTool * KoToolManager::shapeCreatorTool(KoCanvasBase *canvas) const
     return 0;
 }
 
-KoToolBase *KoToolManager::toolById(KoCanvasBase *canvas, const QString id) const
+KoToolBase *KoToolManager::toolById(KoCanvasBase *canvas, const QString &id) const
 {
     Q_ASSERT(canvas);
     foreach(KoCanvasController *controller, d->canvasses.keys()) {
