@@ -1574,6 +1574,11 @@ bool KoDocument::loadNativeFormatFromStore(QByteArray &data)
 bool KoDocument::loadNativeFormatFromStoreInternal(KoStore * store)
 {
     bool oasis = true;
+
+    if (oasis && store->hasFile("manifest.rdf") && d->docRdf) {
+        d->docRdf->loadOasis(store);
+    }
+
     // OASIS/OOo file format?
     if (store->hasFile("content.xml")) {
         store->disallowNameExpansion();
@@ -1621,10 +1626,6 @@ bool KoDocument::loadNativeFormatFromStoreInternal(KoStore * store)
         d->docInfo = new KoDocumentInfo(this);
     }
 
-    if (oasis && store->hasFile("manifest.rdf") && d->docRdf) {
-        d->docRdf->loadOasis(store);
-    }
-
     if (oasis && store->hasFile("VersionList.xml")) {
         KNotification *notify = new KNotification("DocumentHasVersions");
         notify->setText(i18n("Document <i>%1</i> contains several versions. Go to File->Versions to open an old version.", store->urlOfStore().url()));
@@ -1870,7 +1871,7 @@ QString KoDocument::caption() const
     if (documentInfo()) {
         c = documentInfo()->aboutInfo("title");
     }
-    const QString url(prettyPathOrUrl());
+    const QString url(this->url().fileName());
     if (!c.isEmpty() && !url.isEmpty()) {
         c = QString("%1 - %2").arg(c).arg(url);
     }
