@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2006-2007 Cyrille Berger <cberger@cberger.bet
+ *  Copyright (c) 2006,2007,2010 Cyrille Berger <cberger@cberger.bet
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #include "pigment_export.h"
 #include <KoIntegerMaths.h>
 #include "KoChannelInfo.h"
+#include "KoLut.h"
 
 #undef _T
 
@@ -151,6 +152,61 @@ public:
     static const KoChannelInfo::enumChannelValueType channelValueType;
 };
 
+#ifdef Q_CC_MSVC
+// MSVC do not have lrint
+
+const double _double2fixmagic = 68719476736.0*1.5;
+const qint32 _shiftamt        = 16;                    //16.16 fixed point representation,
+
+#if Q_BYTE_ORDER == Q_BIG_ENDIAN
+        #define iexp_                           0
+        #define iman_                           1
+#else
+        #define iexp_                           1
+        #define iman_                           0
+#endif //BigEndian_
+
+inline int float2int(double val)
+{
+    val = val + _double2fixmagic;
+    return ((int*)&val)[iman_] >> _shiftamt; 
+}
+
+inline int float2int(float val)
+{
+    return float2int((double)val);
+}
+
+#else
+#include <cmath>
+
+inline int float2int(float x)
+{
+    return lrintf(x);
+}
+
+inline int float2int(double x)
+{
+    return lrint(x);
+}
+
+#endif
+
+#include <KoLut.h>
+
+template<typename _T_>
+struct KoIntegerToFloat {
+  inline float operator()(_T_ f) const
+  {
+    return f / float(KoColorSpaceMathsTraits<_T_>::max);
+  }
+};
+
+
+extern PIGMENTCMS_EXPORT const Ko::FullLut< KoIntegerToFloat<quint16>, float, quint16> KoUint16ToFloatLut;
+extern PIGMENTCMS_EXPORT const Ko::FullLut< KoIntegerToFloat<quint8>, float, quint8> KoUint8ToFloatLut;
+
+
 /**
  * This class defines some elementary operations used by various color
  * space. It's intended to be generic, but some specialization exists
@@ -210,26 +266,26 @@ template<>
 inline quint8 KoColorSpaceMaths<double, quint8>::scaleToA(double a)
 {
     double v = a * 255;
-    return (quint8)(CLAMP(v, 0, 255));
+    return float2int(CLAMP(v, 0, 255));
 }
 
 template<>
 inline double KoColorSpaceMaths<quint8, double>::scaleToA(quint8 a)
 {
-    return a * (1.0 / 255.0);
+    return KoUint8ToFloatLut(a);
 }
 
 template<>
 inline quint16 KoColorSpaceMaths<double, quint16>::scaleToA(double a)
 {
     double v = a * 0xFFFF;
-    return (quint16)(CLAMP(v, 0, 0xFFFF));
+    return float2int(CLAMP(v, 0, 0xFFFF));
 }
 
 template<>
 inline double KoColorSpaceMaths<quint16, double>::scaleToA(quint16 a)
 {
-    return a * (1.0 / 0xFFFF);
+    return KoUint16ToFloatLut(a);
 }
 
 template<>
@@ -256,26 +312,26 @@ template<>
 inline quint16 KoColorSpaceMaths<float, quint16>::scaleToA(float a)
 {
     float v = a * 0xFFFF;
-    return (quint16)(CLAMP(v, 0, 0xFFFF));
+    return (quint16)float2int(CLAMP(v, 0, 0xFFFF));
 }
 
 template<>
 inline float KoColorSpaceMaths<quint16, float>::scaleToA(quint16 a)
 {
-    return a * (1.0 / 0xFFFF);
+    return KoUint16ToFloatLut(a);
 }
 
 template<>
 inline quint8 KoColorSpaceMaths<float, quint8>::scaleToA(float a)
 {
     float v = a * 255;
-    return (quint8)(CLAMP(v, 0, 255));
+    return (quint8)float2int(CLAMP(v, 0, 255));
 }
 
 template<>
 inline float KoColorSpaceMaths<quint8, float>::scaleToA(quint8 a)
 {
-    return a * (1.0 / 255.0);
+    return KoUint8ToFloatLut(a);
 }
 
 template<>
@@ -328,7 +384,7 @@ inline quint8 KoColorSpaceMaths<half, quint8>::scaleToA(half a)
 template<>
 inline half KoColorSpaceMaths<quint8, half>::scaleToA(quint8 a)
 {
-    return a * (1.0 / 255.0);
+    return a *(1.0 / 255.0);
 }
 template<>
 inline quint16 KoColorSpaceMaths<half, quint16>::scaleToA(half a)
@@ -340,7 +396,7 @@ inline quint16 KoColorSpaceMaths<half, quint16>::scaleToA(half a)
 template<>
 inline half KoColorSpaceMaths<quint16, half>::scaleToA(quint16 a)
 {
-    return a * (1.0 / 0xFFFF);
+    return a *(1.0 / 0xFFFF);
 }
 
 template<>
@@ -364,6 +420,40 @@ inline double KoColorSpaceMaths<half>::clamp(double a)
 
 #endif
 
+//------------------------------ quint8 specialization ------------------------------//
+
+template<>
+inline qint32 KoColorSpaceMaths<quint8>::multiply(qint32 a, qint32 b)
+{
+    return UINT8_MULT(a, b);
+}
+
+template<>
+inline quint8 KoColorSpaceMaths<quint8>::divide(quint8 a, quint8 b)
+{
+    return UINT8_DIVIDE(a, b);
+}
+
+template<>
+inline quint8 KoColorSpaceMaths<quint8>::blend(quint8 a, quint8 b, quint8 c)
+{
+    return UINT8_BLEND(a, b, c);
+}
+
+//------------------------------ quint16 specialization ------------------------------//
+
+template<>
+inline qint64 KoColorSpaceMaths<quint16>::multiply(qint64 a, qint64 b)
+{
+    return UINT16_MULT(a, b);
+}
+
+template<>
+inline quint16 KoColorSpaceMaths<quint16>::divide(quint16 a, quint16 b)
+{
+    return UINT16_DIVIDE(a, b);
+}
+
 //------------------------------ various specialization ------------------------------//
 
 
