@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2007, 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,6 +30,7 @@
 #include <QTextBlock>
 #include <QTextTableCell>
 #include <QHash>
+#include <QWeakPointer>
 
 class KoStyleManager;
 class KoTextBlockData;
@@ -37,6 +38,7 @@ class KoTextShapeData;
 class TextShape;
 class KoChangeTracker;
 class KoImageCollection;
+class ToCGenerator;
 
 /**
  * The document layouter for KoText style docs.
@@ -51,7 +53,7 @@ public:
     virtual void end();
     virtual void reset();
     /// returns true if reset has been called.
-    virtual bool interrupted();
+    virtual bool isInterrupted() const;
     virtual qreal width();
     virtual qreal x();
     virtual qreal y();
@@ -137,6 +139,8 @@ private:
             const QTextLine &line, qreal x1, qreal x2, const int startOfFragmentInBlock,
             const int fragmentToLineOffset) const;
 
+    void updateFrameStack();
+
 private:
     KoStyleManager *m_styleManager;
 
@@ -155,6 +159,8 @@ private:
     KoTextDocumentLayout *m_parent;
     QHash<int, qreal> m_inlineObjectHeights; // maps text-position to whole-line-height of an inline object
     TextShape *m_textShape;
+    QVector<QTextFrame *> m_frameStack;
+    QList<QWeakPointer<ToCGenerator> > m_tocGenerators;
 
     // demoText feature
     bool m_demoText, m_endOfDemoText;
@@ -164,7 +170,7 @@ private:
     int m_currentTabStop; // = n, where we should be looking from the nth tab stop onwards when
     // we decorate the tab for the text of a fragment
     int m_dropCapsNChars, m_dropCapsAffectsNMoreLines;
-    qreal m_dropCapsAffectedLineWidthAdjust, m_y_justBelowDropCaps;
+    qreal m_dropCapsAffectedLineWidthAdjust, m_y_justBelowDropCaps, m_dropCapsPositionAdjust;
 
     QString m_currentMasterPage;
 
@@ -172,8 +178,6 @@ private:
     QTextTableCell m_tableCell;  /**< Current table cell. */
     bool m_restartingAfterTableBreak; /** We are in a re-layout that was a result of a break in a table. */
     bool m_restartingFirstCellAfterTableBreak; /** We are in a re-layout that was a result of a break in a table. only true first cell after */
-
-    KoImageCollection *m_imageCollection;
 };
 
 #endif
