@@ -288,7 +288,7 @@ void ChangeTrackedDeleteCommand::deleteSelection(QTextCursor &selection)
     deleteChangemarker = new KoDeleteChangeMarker(KoTextDocument(document).changeTracker());
     deleteChangemarker->setChangeId(changeId);
     element->setDeleteChangeMarker(deleteChangemarker);
-    deletedFragment = generateDeleteFragment(selection, deleteChangemarker);
+    deletedFragment = KoChangeTracker::generateDeleteFragment(selection, deleteChangemarker);
     element->setDeleteData(deletedFragment);
 
     //Store the position and length. Will be used in updateListChanges()
@@ -323,97 +323,6 @@ void ChangeTrackedDeleteCommand::deleteSelection(QTextCursor &selection)
     }
 }
 
-
-QTextDocumentFragment ChangeTrackedDeleteCommand::generateDeleteFragment(QTextCursor &cursor, KoDeleteChangeMarker *marker)
-{
-    int changeId = marker->changeId();
-    QTextCursor editCursor(cursor);
-    QTextDocument *document = cursor.document();
-    
-    QTextDocument deletedDocument;
-    QTextDocument deleteCursor(&deletedDocument);
-
-    KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(document->documentLayout());
-    
-    for (int i = cursor.anchor();i <= cursor.position(); i++) {
-        if (document->characterAt(i) == QChar::ObjectReplacementCharacter) {
-            editCursor.setPosition(i+1);
-            KoDeleteChangeMarker *testMarker = dynamic_cast<KoDeleteChangeMarker*>(layout->inlineTextObjectManager()->inlineTextObject(editCursor));
-            if (testMarker)
-                editCursor.deletePreviousChar();
-        }
-    }
-
-    QTextCharFormat format;
-    format.setProperty(KoCharacterStyle::ChangeTrackerId, changeId);
-    cursor.mergeCharFormat(format);
-
-    QTextBlock currentBlock = document->findBlock(cursor.anchor());
-    QTextBlock endBlock = document->findBlock(cursor.position()).next();
-
-    // First remove any left-over DeletedList set from previous deletes
-    for (;currentBlock != endBlock; currentBlock = currentBlock.next()) {
-        editCursor.setPosition(currentBlock.position());
-        if (editCursor.currentList()) {
-            if (editCursor.currentList()->format().hasProperty(KoDeleteChangeMarker::DeletedList)) {
-                QTextListFormat format = editCursor.currentList()->format();
-                format.clearProperty(KoDeleteChangeMarker::DeletedList);
-                editCursor.currentList()->setFormat(format);
-            }
-        }
-    }
-
-    currentBlock = document->findBlock(cursor.anchor());
-    endBlock = document->findBlock(cursor.position()).next();
-
-    for (;currentBlock != endBlock; currentBlock = currentBlock.next()) {
-        editCursor.setPosition(currentBlock.position());
-        if (editCursor.currentList()) {
-            if (!editCursor.currentList()->format().hasProperty(KoDeleteChangeMarker::DeletedList)) {
-                bool fullyDeletedList = checkListDeletion(editCursor.currentList(), cursor);
-                QTextListFormat format = editCursor.currentList()->format();
-                format.setProperty(KoDeleteChangeMarker::DeletedList, fullyDeletedList);
-                if (fullyDeletedList) {
-                    KoListStyle::ListIdType listId = ListId(format);
-                    KoList *list = KoTextDocument(document).list(currentBlock);
-                    marker->setDeletedListStyle(listId, list->style());
-                }
-                editCursor.currentList()->setFormat(format);
-            }
-            if (cursor.anchor() <= (currentBlock.position() - 1)) {
-                //Then the list-item has been deleted. Set the block-format to indicate that this is a deleted list-item.
-                QTextBlockFormat blockFormat;
-                blockFormat.setProperty(KoDeleteChangeMarker::DeletedListItem, true);
-                editCursor.mergeBlockFormat(blockFormat);
-            } else {
-                QTextBlockFormat blockFormat;
-                blockFormat.setProperty(KoDeleteChangeMarker::DeletedListItem, false);
-                editCursor.mergeBlockFormat(blockFormat);
-            }
-        }
-    }
-    
-    return cursor.selection();
-}
-
-bool ChangeTrackedDeleteCommand::checkListDeletion(QTextList *list, QTextCursor &cursor)
-{
-    int startOfList = (list->item(0).position() - 1);
-    int endOfList = list->item(list->count() -1).position() + list->item(list->count() -1).length() - 1;
-    if ((cursor.anchor() <= startOfList) && (cursor.position() >= endOfList))
-        return true;
-    else {
-        /***************************************************************************************************/
-        /*                                    Qt Quirk Work-Around                                         */
-        /***************************************************************************************************/
-        if ((cursor.anchor() == (startOfList + 1)) && (cursor.position() > endOfList))
-            return true;
-        /***************************************************************************************************/
-        else
-            return false;
-    }
-} 
-
 void ChangeTrackedDeleteCommand::insertDeleteFragment(QTextCursor &cursor, KoDeleteChangeMarker *marker)
 {
     QTextDocumentFragment fragment =  KoTextDocument(cursor.document()).changeTracker()->elementById(marker->changeId())->getDeleteData();
@@ -450,8 +359,27 @@ void ChangeTrackedDeleteCommand::insertDeleteFragment(QTextCursor &cursor, KoDel
                 } 
                 currentList->add(cursor.block(), deletedListItemLevel);
             }
+        } else if (tempCursor.currentTable()) {
+            QTextTable *deletedTable = tempCursor.currentTable();
+            int numRows = deletedTable->rows();
+            int numColumns = deletedTable->columns();
+            QTextTable *insertedTable = cursor.insertTable(numRows, numColumns, deletedTable->format());
+            for (int i=0; i<numRows; i++) {
+                for (int j=0; j<numColumns; j++) {
+                    tempCursor.setPosition(deletedTable->cellAt(i,j).firstCursorPosition().position());
+                    tempCursor.setPosition(deletedTable->cellAt(i,j).lastCursorPosition().position(), QTextCursor::KeepAnchor);
+                    insertedTable->cellAt(i,j).setFormat(deletedTable->cellAt(i,j).format().toTableCellFormat());
+                    cursor.setPosition(insertedTable->cellAt(i,j).firstCursorPosition().position());
+                    cursor.insertFragment(tempCursor.selection());
+                }
+            }
+            tempCursor.setPosition(deletedTable->cellAt(numRows-1,numColumns-1).lastCursorPosition().position());
+            currentBlock = tempCursor.block();
+            //Move the cursor outside of table
+            cursor.setPosition(cursor.position() + 1);
+            continue;
         } else {
-            // This block does not contain a list. So no special work here. 
+            // This block does not contain a list. So no special work here.
             if (currentBlock != tempDoc.begin())
                 cursor.insertBlock(currentBlock.blockFormat(), currentBlock.charFormat());
         }
@@ -495,6 +423,20 @@ int ChangeTrackedDeleteCommand::fragmentLength(QTextDocumentFragment &fragment)
             deletedListItem = currentBlock.blockFormat().property(KoDeleteChangeMarker::DeletedListItem).toBool();
             if (currentBlock != tempDoc.begin() && deletedListItem)
                 length += 1; //For the Block separator
+        } else if (tempCursor.currentTable()) {
+            QTextTable *deletedTable = tempCursor.currentTable();
+            int numRows = deletedTable->rows();
+            int numColumns = deletedTable->columns();
+            for (int i=0; i<numRows; i++) {
+                for (int j=0; j<numColumns; j++) {
+                    length += 1;
+                    length += (deletedTable->cellAt(i,j).lastCursorPosition().position() - deletedTable->cellAt(i,j).firstCursorPosition().position());
+                }
+            }
+            tempCursor.setPosition(deletedTable->cellAt(numRows-1,numColumns-1).lastCursorPosition().position());
+            currentBlock = tempCursor.block();
+            length += 1;
+            continue;
         } else {
             if (currentBlock != tempDoc.begin())
                 length += 1; //For the Block Separator
